
use egui::{Color32, TextBuffer};


use crate::manifest::{playlist::Playlist, song::{Song, SongType}};

use super::Gui;

#[derive(Debug)]
pub struct GuiSongEditor {
    pub is_open: bool,
    pub is_new_open: bool,
    pub song: (String, String),
    pub ed_url: String,
    pub ed_name: String,
    pub ed_playlist: Option<String>,
    pub ed_type: SongType,
}


impl Gui {
    pub fn draw_song_edit_window(&mut self, ctx: &egui::Context, _: &mut eframe::Frame) {
        let mut save = false;
        
        let (playlist, song_name) = self.song_editor.song.clone();

        let Some(song) = self.manifest.get_song(&playlist, &song_name) else {
            return;
        };
        let song = song.clone();
        
        egui::Window::new("Song editor")
            .open(&mut self.song_editor.is_open)
            .show(ctx, 
            |ui| {

            ui.horizontal(|ui| {
                ui.spacing_mut().item_spacing.x = 0.0;
                ui.label("[");
                ui.hyperlink_to("link", song.get_url().unwrap());
                ui.label("] ");
                ui.colored_label(Color32::LIGHT_BLUE, &playlist);
                ui.label(": ");
                ui.label(&song_name)
            });

            ui.horizontal(|ui| {
                ui.label("Type: ");
                ui.label(&song.get_type().to_string());
            });

            ui.horizontal(|ui| {
                ui.label("Name: ");
                ui.text_edit_singleline(&mut self.song_editor.ed_name);
            });
            ui.horizontal(|ui| {
                ui.label("Url: ");
                ui.text_edit_singleline(&mut self.song_editor.ed_url);
            });

            if ui.button("Save").clicked() {
                save = true;
            }
        });

        if save {
            {
                let Some(song) = self.manifest.get_song_mut(&playlist, &song_name) else {
                    return;
                };
                
                *song.get_url_str_mut() = self.song_editor.ed_url.clone();
            }

            let Some(playlist) = self.manifest.get_playlist_mut(&playlist) else {
                return;
            };


            playlist.remove_song(&song_name);
            playlist.add_song(self.song_editor.ed_name.clone(), song);
            let _ = self.manifest.save(None);
        }
    }
    
    pub fn draw_new_song_window(&mut self, ctx: &egui::Context, _: &mut eframe::Frame) {
        let mut save = false;
        egui::Window::new("New song")
            .open(&mut self.song_editor.is_new_open)
            .show(ctx, |ui| {
                ui.horizontal(|ui| {
                    ui.label("Type: ");
                    egui::ComboBox::from_id_source("new_song_window_type")
                        .selected_text(format!("{:?}", self.song_editor.ed_type))
                        .show_ui(ui, |ui| {
                            ui.selectable_value(&mut self.song_editor.ed_type, SongType::Youtube, "Youtube");
                            ui.selectable_value(&mut self.song_editor.ed_type, SongType::Spotify, "Spotify");
                            ui.selectable_value(&mut self.song_editor.ed_type, SongType::Soundcloud, "Soundcloud");
                        }
                    );    
                });

                ui.horizontal(|ui| {
                    ui.label("Name: ");
                    ui.text_edit_singleline(&mut self.song_editor.ed_name);
                });
                ui.horizontal(|ui| {
                    ui.label("Playlist: ");
                    egui::ComboBox::from_id_source("new_song_window_playlist")
                        .selected_text(format!("{}", self.song_editor.ed_playlist.clone().unwrap()))
                        .show_ui(ui, |ui| {
                            for p in self.manifest.get_playlists().keys() {
                                ui.selectable_value(&mut self.song_editor.ed_playlist, Option::Some(p.clone()), p.as_str());
                            }
                        }
                    );    
                });
                ui.horizontal(|ui| {
                    ui.label("Url: ");
                    ui.text_edit_singleline(&mut self.song_editor.ed_url);
                });

                if ui.button("Save").clicked() {
                    save = true;
                }
        });

        if save {
            let Some(playlist) = self.manifest.get_playlist_mut(&self.song_editor.ed_playlist.clone().unwrap())  else {
                panic!("couldnt find playlist from a preset playlist list????????????");
            };

            playlist.add_song(
                self.song_editor.ed_name.clone(),
                Song::from_url_str(self.song_editor.ed_url.clone()).unwrap().set_type(self.song_editor.ed_type.clone()).clone()
            );


            let _ = self.manifest.save(None);
            self.song_editor.is_new_open = false;
        }
    }

}
